#define rope_arclength
// rope_arclength(a, b, x)
// using the equation y = a*(x-b)^2+c
// assuming a<0
// c is not relevant for the arc length

/*
This formula was calculated with Maple:
y := x->a*(x-b)^2+c;
arclength := int(sqrt(diff(y(x),x)^2+1),x);
simplify(arclength) assuming a<0;
*/

var temp1, temp2;
temp1 = 2*argument0*(argument2-argument1);
temp2 = sqrt(temp1*temp1+1);
// temp2>abs(temp1)
// => temp2-abs(temp1)>0
// => temp2-temp1>temp2-abs(temp1)>0
// => temp2-temp1>0
// => ln(temp2-temp1) exists assuming temp1 is not too large
return temp2*(argument2-argument1)/2-ln(temp2-temp1)/argument0/4;



#define rope_calculate
// rope_calculate(x1, y1, x2, y2, length, iterations)
// returns
//   1 on success
//   2 if the points are too far apart
//   3 if the points are too close
// the result is saved in global.rope_a, global.rope_b, global.rope_c
// using the equation y = a*(x-b)^2+c
var x1, y1, x2, y2, length, a, b, c, len, a1, a2;
x1 = argument0;
y1 = argument1;
x2 = argument2;
y2 = argument3;
length = argument4;

if x1>x2 {
    a = x1; x1 = x2; x2 = a;
    a = y1; y1 = y2; y2 = a;
}
if x2-x1<0.1 {
    return 3;
}

// left bound
a = -0.000001;
b = (a*(x1*x1-x2*x2)+y2-y1)/(2*a*(x1-x2));
len = rope_arclength(a, b, x2)-rope_arclength(a, b, x1);
if len>length {
    return 2;
}
a1 = a;

// right bound
a = -1; // initial guess
while true {
    b = (a*(x1*x1-x2*x2)+y2-y1)/(2*a*(x1-x2));
    len = rope_arclength(a, b, x2)-rope_arclength(a, b, x1);
    if len>length {
        break;
    }
    a1 = a; // move left bound
    a *= 2;
    if a<-1000000 {
        return 3;
    }
}

// bisection method
a2 = a;
repeat argument5 {
    a = (a1+a2)/2;
    b = (a*(x1*x1-x2*x2)+y2-y1)/(2*a*(x1-x2));
    len = rope_arclength(a, b, x2)-rope_arclength(a, b, x1);
    if len>length {
        a2 = a;
    } else {
        a1 = a;
    }
}
a = (a1+a2)/2;

// generate solution
global.rope_a = a;
global.rope_b = (a*(x1*x1-x2*x2)+y2-y1)/(2*a*(x1-x2));
global.rope_c = (2*a*sqr(x1-x2)*(y1+y2)-a*a*sqr(sqr(x1-x2))-sqr(y2-y1))/(a*sqr(x1-x2)*4);
//global.rope_len = rope_arclength(global.rope_a, global.rope_b, x2)-rope_arclength(global.rope_a, global.rope_b, x1);

return 1;



#define draw_rope
// draw_rope(x1, y1, x2, y2, length, iterations, steps, background)

var a, d, e, w, h, tex, xx1, yy1, xx2, yy2, len, len2, rc;

w = background_get_width(argument7);
h = background_get_height(argument7);

a = rope_calculate(argument0, argument1, argument2, argument3, argument4, argument5);
if a=1 {
    
    // reversed?
    if argument0<argument2 {
        e = 1;
    } else {
        e = -1;
    }
    
    // calculate length
    len = 0;
    for(i = 0; i<=argument6; i += 1) {
        xx2 = argument0+(argument2-argument0)*i/argument6;
        yy2 = global.rope_a*sqr(xx2-global.rope_b)+global.rope_c;
        if i!=0 {
            len += point_distance(xx1, yy1, xx2, yy2);
        }
        xx1 = xx2;
        yy1 = yy2;
    }
    
    // draw the rope
    texture_set_repeat(true);
    draw_primitive_begin_texture(pr_trianglestrip, background_get_texture(argument7));
    xx1 = argument0+(argument2-argument0)*0/40;
    yy1 = global.rope_a*sqr(xx1-global.rope_b)+global.rope_c;
    len2 = 0;
    for(i = 0; i<=argument6; i += 1) {
        xx2 = argument0+(argument2-argument0)*i/argument6;
        yy2 = global.rope_a*sqr(xx2-global.rope_b)+global.rope_c;
        if i!=0 {
            len2 += point_distance(xx1, yy1, xx2, yy2);
        }
        rc = 2*global.rope_a*(xx2-global.rope_b);
        d = e*h/2/sqrt(rc*rc+1);
        draw_vertex_texture(xx2-rc*d-0.5, yy2+d-0.5, argument4*len2/len/w, 1);
        draw_vertex_texture(xx2+rc*d-0.5, yy2-d-0.5, argument4*len2/len/w, 0);
        xx1 = xx2;
        yy1 = yy2;
    }
    draw_primitive_end();
    
} else if a=2 {
    xx1 = argument2-argument0;
    yy1 = argument3-argument1;
    d = sqrt(xx1*xx1+yy1*yy1);
    if d>0.001 {
        xx1 *= h/2/d;
        yy1 *= h/2/d;
        texture_set_repeat(true);
        draw_primitive_begin_texture(pr_trianglestrip, background_get_texture(argument7));
        draw_vertex_texture(argument0-yy1-0.5, argument1+xx1-0.5, 0, 1);
        draw_vertex_texture(argument0+yy1-0.5, argument1-xx1-0.5, 0, 0);
        draw_vertex_texture(argument2-yy1-0.5, argument3+xx1-0.5, argument4/w, 1);
        draw_vertex_texture(argument2+yy1-0.5, argument3-xx1-0.5, argument4/w, 0);
        draw_primitive_end();
    }
} else {
    xx1 = (argument0+argument2)/2;
    yy1 = (argument1+argument3+argument4)/2;
    d = h/2;
    e = (argument3-argument1+argument4)/2/w;
    texture_set_repeat(true);
    draw_primitive_begin_texture(pr_trianglestrip, background_get_texture(argument7));
    draw_vertex_texture(argument0-d-0.5, argument1-0.5, 0, 1);
    draw_vertex_texture(argument0+d-0.5, argument1-0.5, 0, 0);
    draw_vertex_texture(xx1-d, yy1, e, 1);
    draw_vertex_texture(xx1+d, yy1, e, 0);
    /*draw_vertex_texture(xx1, yy1+d, e, 1);
    draw_vertex_texture(xx1, yy1-d, e, 0);*/
    draw_vertex_texture(xx1+d, yy1, e, 1);
    draw_vertex_texture(xx1-d, yy1, e, 0);
    draw_vertex_texture(argument2+d-0.5, argument3+0.5, argument4/w, 1);
    draw_vertex_texture(argument2-d-0.5, argument3-0.5, argument4/w, 0);
    draw_primitive_end();
}



